@@ -37,9 +37,9 @@ namespace Bindings {
  * Implements the JS methods of custom object bindings.
  *
  * @author Richard Moore, rich@kde.org
- * @version $Id: customobject_imp.h,v 1.30 2005/02/21 22:42:56 geiseri Exp $
+ * @version $Id: customobject_imp.h 395352 2005-03-06 22:15:34Z bero $
  */
-class CustomObjectImp : public JSProxyImp
+class KJSEMBED_EXPORT CustomObjectImp : public JSProxyImp
 {
 public:
     /** Identifiers for the methods provided by this class. */
