@@ -253,6 +253,7 @@ static int addClassInstances(PyObject *dict,sipClassInstanceDef *ci);
 static int addSingleClassInstance(PyObject *dict,char *name,void *cppPtr,
 				  sipWrapperType *wt,int initflags);
 static int addLicense(PyObject *dict,sipLicenseDef *lc);
+static PyObject *cast(PyObject *self,PyObject *args);
 static PyObject *setTraceMask(PyObject *self,PyObject *args);
 static PyObject *wrapInstance(PyObject *self,PyObject *args);
 static PyObject *unwrapInstance(PyObject *self,PyObject *args);
@@ -266,6 +267,7 @@ static int sipWrapperType_Check(PyObject *op);
 PyMODINIT_FUNC initsip(void)
 {
 	static PyMethodDef methods[] = {
+		{"cast", cast, METH_VARARGS, NULL},
 		{"settracemask", setTraceMask, METH_VARARGS, NULL},
 		{"transfer", transfer, METH_VARARGS, NULL},
 		{"wrapinstance", wrapInstance, METH_VARARGS, NULL},
@@ -410,6 +412,37 @@ static PyObject *transfer(PyObject *self,PyObject *args)
 }
 
 
+/*
+ * Cast an instance to one of it's superclasses by returning a new Python
+ * object with the superclass type wrapping the same C++ instance.
+ */
+static PyObject *cast(PyObject *self,PyObject *args)
+{
+	sipWrapper *w;
+	sipWrapperType *wt;
+	void *addr;
+
+	if (!PyArg_ParseTuple(args,"O!O!:cast",&sipWrapper_Type,&w,&sipWrapperType_Type,&wt))
+		return NULL;
+
+	if (!PyObject_TypeCheck(w, wt))
+	{
+		PyErr_SetString(PyExc_TypeError,"argument 1 of sip.cast() must be an instance of a sub-type of argument 2");
+		return NULL;
+	}
+
+	if ((addr = sip_api_get_cpp_ptr(w,wt)) == NULL)
+		return NULL;
+
+	/*
+	 * We don't put this new object into the map so that the original
+	 * object is always found.  It would also totally confuse the map
+	 * logic.
+	 */
+	return sip_api_new_cpp_to_self(addr,wt,SIP_NOT_IN_MAP);
+}
+
+
 /*
  * Unwrap an instance.
  */
@@ -1579,6 +1612,18 @@ static int parsePass1(sipWrapper **selfp,int *selfargp,int *argsParsedp,
 				break;
 			}
 
+		case 'H':
+			{
+				/* Python callable object or None. */
+ 
+				if (arg == Py_None || PyCallable_Check(arg))
+					*va_arg(va,PyObject **) = arg;
+				else
+					valid = PARSE_TYPE;
+ 
+				break;
+			}
+
 		case 'q':
 			{
 #if defined(SIP_QT_SUPPORT)
@@ -2161,7 +2206,8 @@ static void sip_api_common_dtor(sipWrapper *sipSelf)
 	{
 		SIP_BLOCK_THREADS
 
-		sipOMRemoveObject(&cppPyMap,sipSelf);
+		if (!sipNotInMap(sipSelf))
+			sipOMRemoveObject(&cppPyMap,sipSelf);
 
 		/* This no longer points to anything useful. */
 		sipSelf -> u.cppPtr = NULL;
@@ -2210,8 +2256,8 @@ static sipWrapperType *createType(sipExportedModuleDef *client,
 	sipWrapperType *wt;
 
 	/*
-	 * Create an object corresponding to the full type name and and the
-	 * tail of the type name.
+	 * Create an object corresponding to the full type name and the tail of
+	 * the type name.
 	 */
 	if ((full = PyString_FromString(type -> td_name)) == NULL)
 		goto reterr;
@@ -4087,7 +4133,8 @@ static int sipWrapper_init(sipWrapper *self,PyObject *args,PyObject *kwds)
 	self -> u.cppPtr = sipNew;
 	self -> flags = sipFlags;
 
-	sipOMAddObject(&cppPyMap,self);
+	if (!sipNotInMap(self))
+		sipOMAddObject(&cppPyMap,self);
 
 	return 0;
 }
@@ -4102,7 +4149,7 @@ static void sipWrapper_dealloc(sipWrapper *self)
 	Py_XDECREF(self -> dict);
 
 	/* Ignore any errors.  Don't indirect. */
-	if (self -> u.cppPtr != NULL)
+	if (!sipNotInMap(self) && self -> u.cppPtr != NULL)
 	{
 		sipTypeDef *type = ((sipWrapperType *)self -> ob_type) -> type;
 
