@@ -212,13 +212,13 @@ smokeruby_free(void * p)
 		}
 	} else if (strcmp(className, "QPopupMenu") == 0) {
 		QPopupMenu * item = (QPopupMenu *) o->ptr;
-		if (item->parentWidget(FALSE) != 0) {
+		if (item->parentWidget(false) != 0) {
 			free(o);
 			return;
 		}
 	} else if (isDerivedFromByName(o->smoke, className, "QWidget")) {
 		QWidget * qwidget = (QWidget *) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QWidget"));
-		if (qwidget->parentWidget(TRUE) != 0) {
+		if (qwidget->parentWidget(true) != 0) {
 			free(o);
 			return;
 		}
@@ -314,10 +314,15 @@ resolve_classname(Smoke* smoke, int classId, void * ptr)
 		}
 	} else if (isDerivedFromByName(smoke, smoke->classes[classId].className, "QObject")) {
 		QObject * qobject = (QObject *) smoke->cast(ptr, classId, smoke->idClass("QObject"));
-		const char * classname = qobject->className();
-		Smoke::Index classId = smoke->idClass(classname);
-		if (classId != 0) {
-			return smoke->binding->className(classId);
+		QMetaObject * meta = qobject->metaObject();
+
+		while (meta != 0) {
+			Smoke::Index classId = smoke->idClass(meta->className());
+			if (classId != 0) {
+				return smoke->binding->className(classId);
+			}
+
+			meta = meta->superClass();
 		}
 	} else if (isDerivedFromByName(smoke, smoke->classes[classId].className, "QCanvasItem")) {
 		QCanvasItem * qcanvasitem = (QCanvasItem *) smoke->cast(ptr, classId, smoke->idClass("QCanvasItem"));
@@ -858,7 +863,7 @@ static void marshall_QString(Marshall *m) {
 	     	} else {
                *(m->var()) = rstringFromQString(s);
 	     	}
-	     	if(m->cleanup())
+	     	if(m->cleanup() || m->type().isStack())
 	     	delete s;
          } else {
                 *(m->var()) = Qnil;
